package fishProject;

/*****************************************************************
 * Title:      	BackFin
 * Description:	Class to represent the back fin on a fish.
 * @author 		Bjrn Hkon Horpestad
 * @version 	1.0
*********************************************************************/

public class BackFin {
	
	private float[] backFinCoord;
	private int un = 4;
	private int vn = 9;
	private float factor = 1.0f;
	private float zStart1, zStart2, zStart3, zStart4, zStart5, zStart6, zStart7, zStart8, zStart9;
	private float zCtrp1_1, zCtrp2_1, zCtrp1_2, zCtrp2_2, zCtrp1_3, zCtrp2_3, zCtrp1_4, zCtrp2_4, zCtrp1_5, zCtrp2_5;
	private float zCtrp1_6, zCtrp2_6, zCtrp1_7, zCtrp2_7, zCtrp1_8, zCtrp2_8;
	private float zEnd1, zEnd2, zEnd3, zEnd4, zEnd5, zEnd6, zEnd7, zEnd8, zEnd9;
	
	/**
	*	Constructor
	*
	**/
	public BackFin() { 
		setCoordinates();
		setBackFinCoord(); 
	}
	
	/**
	*	Constructor
	*	@param fac - float that gives the size of the backfin
	**/
	public BackFin(float fac) {
		if(fac != 0.0f)
			factor = fac;
		setCoordinates();
		setBackFinCoord();
	}	
	
	/**
	*	setCoordinates()
	*	Method that sets some of the variables to coordinates
	**/
	public void setCoordinates() {
		zStart1 = 0.0f* factor; zCtrp1_1 = 0.0f * factor; zCtrp2_1 = 0.0f * factor; zEnd1 = 0.0f * factor;
		zStart2 = 0.0f * factor; zCtrp1_2 = 0.05f * factor; zCtrp2_2 = 0.05f * factor; zEnd2 = 0.0f * factor;	
		zStart3 = 0.0f * factor; zCtrp1_3 = 0.1f * factor; zCtrp2_3 = 0.1f * factor; zEnd3 = 0.0f * factor;
		zStart4 = 0.0f * factor; zCtrp1_4 = 0.2f * factor; zCtrp2_4 = 0.2f * factor; zEnd4 = 0.0f * factor;	
		zStart5 = 0.0f * factor; zCtrp1_5 = 0.0f * factor; zCtrp2_5 = 0.0f * factor; zEnd5 = 0.0f * factor;	
		zStart6 = 0.0f * factor; zCtrp1_6 = -0.2f * factor; zCtrp2_6 = -0.2f * factor; zEnd6 = 0.0f * factor;	
		zStart7 = 0.0f * factor; zCtrp1_7 = -0.1f * factor; zCtrp2_7 = -0.1f * factor; zEnd7 = 0.0f * factor;		
		zStart8 = 0.0f * factor; zCtrp1_8 = -0.05f * factor; zCtrp2_8 = -0.05f * factor; zEnd8 = 0.0f * factor;
		zStart9 = 0.0f * factor; zEnd9 = 0.0f;
	}
	
	/**
	*	setBackFinCoord()
	*	Method that sets the control coordinates for the bezier surface that represents the Back fin
	**/
	public void setBackFinCoord() {
  		float tmpBackFin[] = {
			  		-0.5f* factor, -0.8f* factor, zStart1, //1
  				   	0.0f* factor, -0.5f* factor, zCtrp1_1, 
  					0.2f* factor, -0.5f* factor, zCtrp2_1, 
  					0.4f* factor, -0.3f* factor, zEnd1
  					,
  					-0.5f* factor, -0.8f* factor, zStart2, //2
  					0.0f* factor, -0.5f* factor, zCtrp1_2,
  					0.2f* factor, -0.5f* factor, zCtrp2_2,
  					0.4f* factor, -0.3f* factor, zEnd2
  					,
					-1.0f* factor, 0.0f* factor, zStart3, //3 
					0.0f* factor, 0.0f* factor, zCtrp1_3,
					0.2f* factor, 0.0f* factor, zCtrp2_3,
					0.4f* factor, 0.0f* factor, zEnd3
  					,
  					-1.0f* factor, 1.0f* factor, zStart4, //4
  					0.0f* factor, 1.5f* factor, zCtrp1_4,
  					0.2f* factor, 1.5f* factor, zCtrp2_4, 
  					0.4f* factor, 1.0f* factor, zEnd4
  					,
  					-1.0f* factor, 1.0f* factor, zStart5, //5
  					-0.2f* factor, 2.4f* factor, zCtrp1_5,
  					0.2f* factor, 2.4f* factor, zCtrp2_5, 
  					0.4f* factor, 1.0f* factor, zEnd5
  					,
  					-1.0f* factor, 1.0f* factor, zStart6, //6
  					0.0f* factor, 1.5f* factor, zCtrp1_6,
  					0.2f* factor, 1.5f* factor, zCtrp2_6, 
  					0.4f* factor, 1.0f* factor, zEnd6
  					,
 					-1.0f* factor, 0.0f* factor, zStart7, //7 
					0.0f* factor, 0.0f* factor, zCtrp1_7,
					0.2f* factor, 0.0f* factor, zCtrp2_7,
					0.4f* factor, 0.0f* factor, zEnd7 					
					,
  					-0.5f* factor, -0.8f* factor, zStart8, //8
  					0.0f* factor, -0.5f* factor, zCtrp1_8,
  					0.2f* factor, -0.5f* factor, zCtrp2_8,
  					0.4f* factor, -0.3f* factor, zEnd8
  					,
  					-0.5f* factor, -0.8f* factor, zStart9, //9
  				   	0.0f* factor, -0.5f* factor, zCtrp1_1, 
  					0.2f* factor, -0.5f* factor, zCtrp2_1, 
  					0.4f* factor, -0.3f* factor, zEnd9					
				   };	
  						
  		backFinCoord = tmpBackFin;
	}
	
	/**
	*	getBackFinCoord()
	*	@return float[] that contains the coordinates to the backFin bezier.
	**/
	public float[] getBackFinCoord() { return backFinCoord; }
	
	/**
	*	getUN()
	*	@return int un
	**/
	public int getUN() { return un; }
	/**
	*	getVN()
	*	@return int vn
	**/	
	public int getVN() { return vn; }
	
	/**
	*	moveBackFin()
	*	Method that moves the back fin during the animation. 
	*	The control points are divided since there shouldn't be as big values as for the body
	*	@param newZ - float with the new Z values
	**/	
	public void moveBackFin(float newZ) {
		zStart1 = newZ/5.0f; zCtrp1_1 = newZ/5.0f; zCtrp2_1 = newZ/5.0f; zEnd1 = newZ/5.0f;
		zStart2 = newZ/7.0f; zCtrp1_2 = newZ/7.0f; zCtrp2_2 = newZ/7.0f; zEnd2 = newZ/7.0f;
		zStart3 = newZ/9.0f; zCtrp1_3 = newZ/9.0f; zCtrp2_3 = newZ/9.0f; zEnd3 = newZ/9.0f;
		zStart7 = newZ/9.0f; zCtrp1_7 = newZ/9.0f; zCtrp2_7 = newZ/9.0f; zEnd7 = newZ/9.0f;
		zStart8 = newZ/7.0f; zCtrp1_8 = newZ/7.0f; zCtrp2_8 = newZ/7.0f; zEnd8 = newZ/7.0f;
		zStart9 = newZ/5.0f;
		setBackFinCoord();
	}
}