package fishProject;

/**********************************
 * Title:      	BezierCurve
 * Description:	Class to make the bezierCurve that the object will move along.
 * @author 		Bjrn Hkon Horpestad
 * @version 	1.0
***************************************/

public class BezierCurve {

	private float[][] btab;
	private float[][] d_btab;
	private float[][] dd_btab;
	private float[][] bez;
	private float[][] d_bez;
	private float[][] dd_bez;
	private float[] ctrPts, bT, bZ;
	private int order = 10;
	private Vector3D bgp0, bgp1, bgp2, bgp3, bgp4;
	private static float dy1=-1.0f;
	private	static float dy2=0.5f;
	private static float dz0=-0.5f;
	private static float dz3=1.0f;
	
	/**
	*	BezierCurve()
	*	@param ant - int that says how many t's ther shall be.
	**/
	public BezierCurve(int ant) {
	
		setOrder(ant);
		
		btab = new float[4][order];
		d_btab = new float[4][order];
		dd_btab = new float[4][order];
		bez = new float[order][4];
		d_bez = new float[order][4];
		dd_bez = new float[order][4];
		
		bgp0 = new Vector3D(-35.0f, -2.5f, -5.0f);
		bgp1 = new Vector3D(-10.0f, 0.5f, 30.0f);
		bgp2 = new Vector3D(10.0f, -1.5f, -30.0f);
		bgp3 = new Vector3D(35.0f, 2.5f, 5.0f);
		

		// These are independent of controlpoints.
		// Bezier
		setBtab();
		// Derivated of Bezier
		setDBtab();
		// Double derivated of Bezier
		setDDBtab();

		setCtrPts();
	
		// done here and each time controlpoints are
		// changed, on timer-event
		makeBezier(bgp0,bgp1,bgp2,bgp3);
		make_d_Bezier(bgp0,bgp1,bgp2,bgp3);
		make_dd_Bezier(bgp0,bgp1,bgp2,bgp3);
	

	}
	
	/**
	*	changeBezier()
	*	Method that changes the bezier so that the fish swims in an other direction
	**/
	public void changeBezier() {
		bgp0 = new Vector3D(-35.0f, -2.5f, -40.0f);
		bgp1 = new Vector3D(-10.0f, 0.5f, 30.0f);
		bgp2 = new Vector3D(0.0f, -1.5f, -30.0f);
		bgp3 = new Vector3D(10.0f, 2.5f, 50.0f);
		
		// Bezier
		setBtab();
		// Derivated of Bezier
		setDBtab();
		// Double derivated of Bezier
		setDDBtab();

		setCtrPts();
	
		// done here and each time controlpoints are
		// changed, on timer-event
		makeBezier(bgp0,bgp1,bgp2,bgp3);
		make_d_Bezier(bgp0,bgp1,bgp2,bgp3);
		make_dd_Bezier(bgp0,bgp1,bgp2,bgp3);
	}

	// get and set methods
	/**
	*	setOrder()
	*	@param i - int new order value
	**/
	public void setOrder(int i) { order = i; }
	/**
	*	getOrder()
	*	@return int order
	**/
	public int getOrder() { return order; }
	
	/**
	*	setCtrPts
	*	Method that sets the control points for the bezierCurve.
	**/
	public void setCtrPts() {
		float tmpCtrPts[] = {
							bgp0.getX(), bgp0.getY(), bgp0.getZ(),
							bgp1.getX(), bgp1.getY(), bgp1.getZ(),
							bgp2.getX(), bgp2.getY(), bgp2.getZ(),
							bgp3.getX(), bgp2.getY(), bgp3.getZ()
							};
		ctrPts = tmpCtrPts;	
	}
	
	/**
	*	getCtrPts()
	*	@return float[] ctrPts, returns the ctrPts to the bezierCurve
	**/
	public float[] getCtrPts() { return ctrPts; }

	/**
	*	setBtab()
	*	Method that sets the btab with weights T x M ( independant of controlpoints)
	*	for cubic(3rd order) curve. 
	*	Bezier: btab(position)
	**/
	public void setBtab() {
		int tix=0;
		while(tix<order) {
			float t=(1.0f/(1.0f*order))*(1.0f*tix);
			btab[0][tix]=(1-t)*(1-t)*(1-t);
			btab[1][tix]=3.0f*t*(1-t)*(1-t);
			btab[2][tix]=3.0f*t*t*(1-t);
			btab[3][tix]=t*t*t;
			tix++;
		}	
	}

	/**
	*	getBtab()
	*	@return float[][] btab
	**/	
	public float[][] getBtab() { return btab; }

	/**
	*	setDBtab()
	* 	Method that sets the derivative of Bezier: d_btab (direction)
	**/
	public void setDBtab(){
		int tix=0;
		while(tix<order) {	
			float t=(1.0f/(1.0f*order))*(1.0f*tix);
			d_btab[0][tix]=-3.0f*t*t+6.0f*t-3.0f;
			d_btab[1][tix]=9.0f*t*t-12.0f*t+3.0f;
			d_btab[2][tix]=-9.0f*t*t+6.0f*t;
			d_btab[3][tix]=3.0f*t*t;
			tix++;
		}
	}
	
	/**
	*	setDDBtab()
	* 	Method that sets the double derivative of Bezier: (change of direction)
	**/
	public void setDDBtab() {
		int tix=0;
		while(tix<order) {
			float t=(1.0f/(1.0f*order))*(1.0f*tix);
			dd_btab[0][tix]=-6.0f*t+6.0f;
			dd_btab[1][tix]=18.0f*t-12.0f;
			dd_btab[2][tix]=-18.0f*t+6.0f;
			dd_btab[3][tix]=6.0f*t;
			tix++;
		}		
	}

//**************************************************************
// calculate tables for function, derivative and doublederivative
// The respective weigth-functions are already prepared in
// btab, d_btab and dd_btab

	/**
	*	makeBezier()
	*	Method that calculates table for function
	*	@param p0 - Vector3D
	*	@param p1 - Vector3D
	*	@param p2 - Vector3D
	*	@param p3 - Vector3D
	**/
	public void makeBezier(Vector3D p0,Vector3D p1,Vector3D p2,Vector3D p3) {
		int ix=0;
		while(ix<order) {
			bez[ix][0]=p0.getX()*btab[0][ix]+p1.getX()*btab[1][ix]+p2.getX()*btab[2][ix]+p3.getX()*btab[3][ix];
			bez[ix][1]=p0.getY()*btab[0][ix]+p1.getY()*btab[1][ix]+p2.getY()*btab[2][ix]+p3.getY()*btab[3][ix];
			bez[ix][2]=p0.getZ()*btab[0][ix]+p1.getZ()*btab[1][ix]+p2.getZ()*btab[2][ix]+p3.getZ()*btab[3][ix];
		
			ix++;
		}	
	}

	/**
	*	make_d_Bezier()
	*	Method that calculates table for derivate
	*	@param p0 - Vector3D
	*	@param p1 - Vector3D
	*	@param p2 - Vector3D
	*	@param p3 - Vector3D
	**/
	public void make_d_Bezier(Vector3D p0,Vector3D p1,Vector3D p2,Vector3D p3) {
		int ix=0;
		while(ix<order) {
			d_bez[ix][0]=p0.getX()*d_btab[0][ix]+p1.getX()*d_btab[1][ix]+p2.getX()*d_btab[2][ix]+p3.getX()*d_btab[3][ix];
			d_bez[ix][1]=p0.getY()*d_btab[0][ix]+p1.getY()*d_btab[1][ix]+p2.getY()*d_btab[2][ix]+p3.getY()*d_btab[3][ix];
			d_bez[ix][2]=p0.getZ()*d_btab[0][ix]+p1.getZ()*d_btab[1][ix]+p2.getZ()*d_btab[2][ix]+p3.getZ()*d_btab[3][ix];
		
			ix++;
		}
	}

	/**
	*	make_dd_Bezier()
	*	Method that calculates table for double derivated
	*	@param p0 - Vector3D
	*	@param p1 - Vector3D
	*	@param p2 - Vector3D
	*	@param p3 - Vector3D
	**/
	public void make_dd_Bezier(Vector3D p0,Vector3D p1,Vector3D p2,Vector3D p3) {
		int ix=0;
		while(ix<order)
		{	
			dd_bez[ix][0]=p0.getX()*dd_btab[0][ix]+p1.getX()*dd_btab[1][ix]+p2.getX()*dd_btab[2][ix]+p3.getX()*dd_btab[3][ix];
			dd_bez[ix][1]=p0.getY()*dd_btab[0][ix]+p1.getY()*dd_btab[1][ix]+p2.getY()*dd_btab[2][ix]+p3.getY()*dd_btab[3][ix];
			dd_bez[ix][2]=p0.getZ()*dd_btab[0][ix]+p1.getZ()*dd_btab[1][ix]+p2.getZ()*dd_btab[2][ix]+p3.getZ()*dd_btab[3][ix];

			ix++;
		}
	}
	
	/**
	*	getBezAtPos()
	*	Method that returns the value at position i j
	*	@param i - int position
	*	@param j - int position
	*	@return float at position i j
	**/
	public float getBezAtPos(int i, int j) { return bez[i][j]; }
	/**
	*	getDBezAtPos()
	*	Method that returns the value at position i j
	*	@param i - int position
	*	@param j - int position
	*	@return float at position i j
	**/	
	public float getDBezAtPos(int i, int j) { return d_bez[i][j]; }
	/**
	*	getDDBezAtPos()
	*	Method that returns the value at position i j
	*	@param i - int position
	*	@param j - int position
	*	@return float at position i j
	**/
	public float getDDBezAtPos(int i, int j) { return dd_bez[i][j]; }
	
	/**
	*	moveBezier()
	*	Method that moves the Bezier around the screen. 
	**/
	public void moveBezier() {			
				// adjust controlpoints of 
				// the Bezierfunction and force update
				// actual values to change are:
				float tmp = 0.0f;

				tmp = bgp1.getY();
				bgp1.setY(tmp += dy1);
				tmp = bgp2.getY();
				bgp2.setY(tmp += dy2);
				if((bgp1.getY() > 30.0f) || (bgp1.getY() < -30.0f)) 
					dy1*=-1.0f;
				if((bgp2.getY() > 30.0f)||(bgp2.getY() < -30.0f)) 
					dy2*=-1.0f;

				tmp = bgp0.getZ() ;
				bgp0.setZ(tmp += dz0);
				tmp = bgp3.getZ();
				bgp3.setZ(tmp += dz3);
				if((bgp0.getZ() > 20.0f)||(bgp0.getZ() < -20.0f)) 
					dz0*=-1.0f;
				if((bgp3.getZ() > 20.0f)||(bgp3.getZ() < -20.0f)) 
					dz3*=-1.0f;

				// mark a point moving back and forth on 
				// the bezier

				makeBezier(bgp0,bgp1,bgp2,bgp3);
				make_d_Bezier(bgp0,bgp1,bgp2,bgp3);
				make_dd_Bezier(bgp0,bgp1,bgp2,bgp3);
			
		}
	
}

