package fishProject;

/*********************************************************
 * Title:      	LeftSideFin
 * Description:	Class to represent the Left side fin.
 * @author 		Bjrn Hkon Horpestad
 * @version 	1.0
*********************************************************/

public class LeftSideFin{
	
	private float[] sideFin;
	private float factor = 1.0f;
	private int un = 2;
	private int vn = 4;
	private float zStart1, zStart2;
	private float zCtrp1_1, zCtrp2_1, zCtrp1_2, zCtrp2_2;
	private float zEnd1, zEnd2;
	
	/**
	*	LeftSideFin()
	*	Constructor
	**/
	public LeftSideFin(){
		setCoordinates();
		setLeftSideCoord();
	}
	
	/**
	*	LeftSideFin()
	*	@param fac - float that representing the change in size on the rightside fin
	**/	
	public LeftSideFin(float fac) {
		if(fac > 0.0f)
			factor = fac;
		setCoordinates();
		setLeftSideCoord();
	}
	
	/**
	*	setCoorinates()
	*	Method that sets the coordinates on the variables that are changed by the move method.
	**/
	private void setCoordinates() {
		zStart1 = -0.3f * factor; zCtrp1_1 = 0.0f * factor; zCtrp2_1 = -0.3f* factor; zEnd1 = 0.0f * factor;	
		zStart2 = -0.3f * factor; zCtrp1_2 = 0.0f * factor; zCtrp2_2 = -0.3f* factor; zEnd2 = 0.0f * factor;
	}
	
	/**
	*	setLeftSideCoord()
	*	Method that sets all the coordinates for the bezier surface representing the left side fin
	**/
	public void setLeftSideCoord() {
		float tmpSideCoord[] =  {
							 0.0f * factor, 0.1f * factor, zStart1,
							 1.0f * factor, 0.1f * factor, zCtrp1_1,
							 0.0f * factor, 0.1f * factor, zCtrp2_1,
							 1.0f * factor, 0.1f * factor, zEnd1
							,
							 0.2f * factor, -0.4f * factor, zStart2,
							 1.0f * factor, 0.0f * factor, zCtrp1_2,
							 0.2f * factor, -0.6f * factor, zCtrp2_2,							
							 1.0f * factor, -0.1f * factor, zEnd2
						};
   		sideFin = tmpSideCoord;
	}
	
	/**
	*	getSideFinCoord()
	*	@return float[] conating the coordinates for the sideFin
	**/
	public float[] getSideFinCoord() { return sideFin;	}
	
	/**
	*	getUN()
	*	@return int un
	**/	
	public int getUN() { return un; }
	/**
	*	getVN()
	*	@return int vn
	**/	
	public int getVN() { return vn; }
	
	/**
	*	moveSideFin()
	*	Method that moves the side fin. 
	*	@param newZ - float that tells how much the change is
	**/	
	public void moveSideFin(float newZ) {
		zStart1 = (-0.3f * factor) + (newZ/8.0f);
		zCtrp2_1 = (-0.3f * factor) + (newZ/8.0f);
		zStart2 = (-0.3f * factor) + (newZ/8.0f);
		zCtrp2_2 = (-0.3f* factor) + (newZ/8.0f);
		setLeftSideCoord();	
	}
}